import json

none = "d3043820717d74d9a17694c176d39733"

# region Subscription
class Subscription:
	"""
	# Arguments
	resource_id: str
	protocol: str
	endpoint: str
	event_type: str
	event_format: dict
	"""
	def __init__(
		self,
		resource_id=none,
		protocol=none,
		endpoint=none,
		event_type=none,
		event_format=none):

		self.resource_id = resource_id
		self.protocol = protocol
		self.endpoint = endpoint
		self.event_type = event_type
		self.event_format = event_format

# endregion


class SubscriptionRequest:
    def __init__(self, subscription):
        self.subscription = subscription

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)




import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Prometheus
class Prometheus:
    """
    # Arguments
    address : str
    """

    def __init__(
            self,
            address: str = none):
        self.address = address
# endregion


# region Datadog
class Datadog:
    """
    # Arguments
    address : str
    api_key : str
    app_key : str
    """

    def __init__(
            self,
            address: str = none,
            api_key: str = none,
            app_key: str = none):
        self.address = address
        self.api_key = api_key
        self.app_key = app_key
# endregion


# region New Relic
class NewRelic:
    """
    # Arguments
    account_id : str
    base_url_nerd_graph : str
    base_url_rest : str
    personal_api_key : str
    region : str
    """

    def __init__(
            self,
            account_id: str = none,
            base_url_nerd_graph: str = none,
            base_url_rest: str = none,
            personal_api_key: str = none,
            region: str = none):
        self.account_id = account_id
        self.base_url_nerd_graph = base_url_nerd_graph
        self.base_url_rest = base_url_rest
        self.personal_api_key = personal_api_key
        self.region = region
# endregion


# region CloudWatch
class CloudWatch:
    """
    # Arguments
    iam_arn : str
    """
    def __init__(
            self,
            iam_arn: str = none):
        self.iam_arn = iam_arn
# endregion


# region Verification Provider
class VerificationProvider:
    """
    # Arguments
    name : str
    cluster_ids : List[str]
    prometheus : Prometheus
    datadog : Datadog
    newRelic : NewRelic
    cloudWatch : CloudWatch
    """

    def __init__(
            self,
            name: str = none,
            cluster_ids: List[str] = none,
            prometheus: Prometheus = none,
            datadog: Datadog = none,
            new_relic: NewRelic = none,
            cloud_watch: CloudWatch = none):
        self.name = name
        self.cluster_ids = cluster_ids
        self.prometheus = prometheus
        self.datadog = datadog
        self.new_relic = new_relic
        self.cloud_watch = cloud_watch
# endregion


# region Client Requests
class CreateVerificationProviderRequest:
    """
    # Arguments
    verification_provider : VerificationProvider
    """
    def __init__(self, verification_provider: VerificationProvider):
        self.verification_provider = verification_provider

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion



import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region ClusterNotification
class ClusterNotification:
    """
    # Arguments
    minutes_without_heartbeat: int
    providers: List[str]
    """

    def __init__(
            self,
            minutes_without_heartbeat: int = none,
            providers: List[str] = none):
        self.minutes_without_heartbeat = minutes_without_heartbeat
        self.providers = providers
# endregion


# region Client Requests
class UpdateClusterNotification:
    def __init__(self, cluster_notification: ClusterNotification):
        self.cluster_notification = cluster_notification

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion



import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"

# region Strategy
class Strategy:
    """
    # Arguments
    draining_timeout: int
    availability_vs_cost: str
    risk: int
    fallback_to_od: bool
    """
    def __init__(
            self,
            draining_timeout:int = none,
            availability_vs_cost:str = none,
            risk:int = none,
            fallback_to_od:bool = none
            ):
        self.draining_timeout = draining_timeout
        self.availability_vs_cost = availability_vs_cost
        self.risk = risk
        self.fallback_to_od = fallback_to_od
# endregion

# region Capacity
class Capacity:
    """
    # Arguments
    minimum: int
    maximum: int
    target: int
    unit: str
    """
    def __init__(self, minimum:int=none, maximum:int=none, target:int=none, unit:str=none):

        self.minimum = minimum
        self.maximum = maximum
        self.target = target
        self.unit = unit

# endregion

# region Weight
class Weight:
    """
    # Arguments
    instance_type: str
    weighted_capacity: int
    """
    def __init__(self, instance_type: str=none, weighted_capacity: int=none):
        self.instance_type = instance_type
        self.weighted_capacity = weighted_capacity

# endregion

# region Instance Types
class InstanceTypes:
    """
    # Arguments
    on_demand: str
    spot: List[str]
    weights: List[Weight]
    preferred_spot: List[str]
    """
    def __init__(
            self,
            on_demand: str = none,
            spot: List[str] = none,
            weights: List[Weight] = none,
            preferred_spot: List[str] = none):

        self.on_demand = on_demand
        self.spot = spot
        self.weights = weights
        self.preferred_spot = preferred_spot

# endregion

# region Image
class Image:
    """
    # Arguments
    id: str
    """
    def __init__(
        self,
        id: str=none):
        self.id = id

# endregion

# region Tag
class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """
    def __init__(self, tag_key: str=none, tag_value: str=none):

        self.tag_key = tag_key
        self.tag_value = tag_value

# endregion

# region Launch Specification
class LaunchSpecification:
    """
    # Arguments
    security_group_ids: List[str]
    image_id: str
    monitoring: bool
    key_pair: str
    user_data: str
    images: List[Image]
    tags: List[Tag]
    """
    def __init__(
            self,
            security_group_ids: List[str] = none,
            image_id: str = none,
            monitoring: bool = none,
            key_pair: str = none,
            user_data: str = none,
            images: List[Image] = none,
            tags: List[Tag] = none):

        self.security_group_ids = security_group_ids
        self.monitoring = monitoring
        self.image_id = image_id
        self.key_pair = key_pair
        self.user_data = user_data
        self.images =images
        self.tags = tags

# endregion


# region Compute
class Compute:
    """
    # Arguments
    launch_specification: LaunchSpecification
    instance_types: InstanceTypes
    subnet_ids: List[str]
    """
    def __init__(
            self,
            launch_specification: LaunchSpecification = none,
            instance_types: InstanceTypes = none,
            subnet_ids: List[str] = none):

        self.subnet_ids = subnet_ids
        self.instance_types = instance_types
        self.launch_specification = launch_specification

# endregion

# region HPC
class HPC:
    """
    # Arguments
    name: str
    description: str
    region: str
    capacity: Capacity
    strategy: Strategy
    compute: Compute
    """
    def __init__(
            self,
            name: str = none,
            description: str=none,
            region: str=none,
            capacity: Capacity = none,
            strategy: Strategy = none,
            compute:Compute = none,
            ):

        self.name = name
        self.description = description
        self.region = region
        self.capacity = capacity
        self.strategy = strategy
        self.compute = compute

# endregion

class LSFClusterCreationRequest:
    def __init__(self, lsf_cluster):
        self.lsf_cluster = lsf_cluster

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class LSFClusterDeletionRequest:
    def __init__(self, lsf_cluster):
        self.lsf_cluster = lsf_cluster

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
    
class LSFClusterUpdateRequest:
    def __init__(self, lsf_cluster):
        self.lsf_cluster = lsf_cluster

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



import json

none = "d3043820717d74d9a17694c176d39733"

# region Task
class Task:
    """
    # Arguments
    name: str
    description: str
    state: str
    policies: list[Policy]
    instances: list[Instance]
    """
    def __init__(
            self,
            name=none,
            description=none,
            state=none,
            policies=none,
            instances=none):

        self.name = name
        self.description = description
        self.state = state
        self.policies = policies
        self.instances = instances

class Policy:
    """
    # Arguments
    cron: str
    action: str
    """
    def __init__(
            self,
            cron=none,
            action=none):

        self.cron = cron
        self.action = action

class Instance:
    """
    # Arguments
    vm_name: str
    resource_group_name: str
    """
    def __init__(
            self,
            vm_name=none,
            resource_group_name=none):

        self.vm_name = vm_name
        self.resource_group_name = resource_group_name

class TaskCreationRequest:
    def __init__(self, task):
        self.name = task.name
        self.description = task.description
        self.state = task.state
        self.policies = task.policies
        self.instances = task.instances

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


from spotinst_sdk2.session import Session

from spotinst_sdk2.clients.managed_instance import *
from spotinst_sdk2.clients.elastigroup import *
from spotinst_sdk2.clients.stateful_node import *
from spotinst_sdk2.clients.ocean import *
from spotinst_sdk2.clients.admin import *
from spotinst_sdk2.clients.functions import *
from spotinst_sdk2.clients.mcs import *
from spotinst_sdk2.clients.mlb import *
from spotinst_sdk2.clients.mrscaler import *
from spotinst_sdk2.clients.subscription import *
from spotinst_sdk2.clients.setup import *
from spotinst_sdk2.clients.ocean_cd import *
from spotinst_sdk2.clients.hpc import *

class SpotinstSession:
    def __init__(self,
                 base_url=None,
                 auth_token=None,
                 account_id=None,
                 profile=None,
                 credentials_file=None):
        self.session = Session(base_url=base_url,
                               auth_token=auth_token,
                               account_id=account_id,
                               profile=profile,
                               credentials_file=credentials_file)

    def client(self, service, print_output=True, log_level=None, user_agent=None, timeout=None):
        switcher = {
            "admin": AdminClient(session=self.session, print_output=print_output, log_level=log_level,
                                 user_agent=user_agent, timeout=timeout),
            "setup_aws": SetupAWSClient(session=self.session, print_output=print_output, log_level=log_level,
                                        user_agent=user_agent, timeout=timeout),
            "setup_azure": SetupAzureClient(session=self.session, print_output=print_output, log_level=log_level,
                                            user_agent=user_agent, timeout=timeout),
            "setup_gcp": SetupGCPClient(session=self.session, print_output=print_output, log_level=log_level,
                                        user_agent=user_agent, timeout=timeout),
            "elastigroup_aws": ElastigroupAwsClient(session=self.session, print_output=print_output,
                                                    log_level=log_level, user_agent=user_agent, timeout=timeout),
            "elastigroup_azure": ElastigroupAzureClient(session=self.session, print_output=print_output,
                                                        log_level=log_level, user_agent=user_agent, timeout=timeout),
            "elastigroup_azure_v3": ElastigroupAzureV3Client(session=self.session, print_output=print_output,
                                                             log_level=log_level, user_agent=user_agent,
                                                             timeout=timeout),
            "elastigroup_gcp": ElastigroupGcpClient(session=self.session, print_output=print_output,
                                                    log_level=log_level, user_agent=user_agent, timeout=timeout),
            "stateful_node_azure": StatefulNodeAzureClient(session=self.session, print_output=print_output,
                                                           log_level=log_level, user_agent=user_agent, timeout=timeout),
            "functions": FunctionsClient(session=self.session, print_output=print_output, log_level=log_level,
                                         user_agent=user_agent, timeout=timeout),
            "mcs": McsClient(session=self.session, print_output=print_output, log_level=log_level,
                             user_agent=user_agent, timeout=timeout),
            "mlb": MlbClient(session=self.session, print_output=print_output, log_level=log_level,
                             user_agent=user_agent, timeout=timeout),
            "mrScaler_aws": MrScalerAwsClient(session=self.session, print_output=print_output, log_level=log_level,
                                              user_agent=user_agent, timeout=timeout),
            "ocean_aws": OceanAwsClient(session=self.session, print_output=print_output, log_level=log_level,
                                        user_agent=user_agent, timeout=timeout),
            "ocean_azure": OceanAzureClient(session=self.session, print_output=print_output, log_level=log_level,
                                        user_agent=user_agent, timeout=timeout),                            
            "oceancd": OceanCDClient(session=self.session, print_output=print_output, log_level=log_level,
                                      user_agent=user_agent, timeout=timeout),
            "managed_instance_aws": ManagedInstanceAwsClient(session=self.session, print_output=print_output,
                                                             log_level=log_level, user_agent=user_agent,
                                                             timeout=timeout),
            "subscription": SubscriptionClient(session=self.session, print_output=print_output, log_level=log_level,
                                               user_agent=user_agent, timeout=timeout),
            "hpc": HPCAwsClient(session=self.session, print_output=print_output,
                                                    log_level=log_level, user_agent=user_agent, timeout=timeout)
        }

        return switcher.get(service, "Invalid Service")



import json

none = "d3043820717d74d9a17694c176d39733"

# region EMR
class UserMapping:
	"""
	# Arguments
	user_email: str
	account_id: str
	role: str
	"""
	def __init__(
		self,
		user_email=none,
		account_id=none,
		role=none):

		self.user_email = user_email
		self.account_id = account_id
		self.role = role


class UserMappingRequest:
    def __init__(self, mappings):
        self.mappings = mappings

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)






import json

none = "d3043820717d74d9a17694c176d39733"

# region EMR
class BlueGreenDeployment:
	"""
	# Arguments
	timeout: int
	tags: List[Tag]
	deployment_groups: List[DeploymentGroup]
	"""
	def __init__(
		self,
		timeout=none,
		tags=none,
		deployment_groups=none):

		self.timeout = timeout
		self.tags = tags
		self.deployment_groups = deployment_groups

# endregion

class Tag:
	"""
	# Arguments
	tag_get: str
	tag_value: str
	"""
	def __init__(
		self,
		tag_key=none,
		tag_value=none):

		self.tag_key = tag_key
		self.tag_value = tag_value

class DeploymentGroup:
	"""
	# Arguments
	application_name: str
	deployment_group_name: str
	"""
	def __init__(
		self,
		application_name=none,
		deployment_group_name=none):

		self.application_name = application_name
		self.deployment_group_name = deployment_group_name



class BlueGreenDeploymentRequest:
    def __init__(self, deployment):
        self.deployment = deployment

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)





import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.ocean_cd as oceancd_cluster
import spotinst_sdk2.models.ocean_cd.verification_provider as oceancd_vp
import spotinst_sdk2.models.ocean_cd.verification_template as oceancd_vt
from spotinst_sdk2.models.ocean_cd import strategy
from spotinst_sdk2.models.ocean_cd import rollout_spec
from spotinst_sdk2.models.ocean_cd import rollout


class OceanCDClient(Client):
    __base_oceancd_cluster_url = "/ocean/cd/cluster"
    __base_oceancd_vp_url = "/ocean/cd/verificationProvider"
    __base_oceancd_vt_url = "/ocean/cd/verificationTemplate"
    __base_strategy_url = "/ocean/cd/strategy"
    __base_rollout_spec_url = '/ocean/cd/rolloutSpec'
    __base_rollout_url = '/ocean/cd/rollout'
    __base_workload_url = '/ocean/cd/workload'

    def get_oceancd_cluster(self, cluster_id: str):
        """
        Get an existing Ocean CD cluster.

        # Arguments
        cluster_id (String): OceanCD Cluster ID

        # Returns
        (Object): OceanCD Cluster API response
        """
        response = self.send_get(
            url=self.__base_oceancd_cluster_url + "/" + cluster_id,
            entity_name="oceancdCluster")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_oceancd_clusters(self):
        """
        List all Ocean CD clusters.

        # Returns
        (List): List of Ocean CD CLuster API response
        """
        response = self.send_get(
            url=self.__base_oceancd_cluster_url,
            entity_name="oceancdCluster")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def update_oceancd_cluster(self, cluster_notification: oceancd_cluster.ClusterNotification,
                               cluster_id: str):
        """
        Update Ocean CD cluster notification settings.

        # Arguments
        cluster_id (String): OceanCD Cluster ID
        cluster_notification (ClusterNotification): Cluster Notification

        # Returns
        (Boolean): Response Status
        """
        request = oceancd_cluster.UpdateClusterNotification(cluster_notification)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_oceancd_cluster_url + "/" + cluster_id,
            entity_name="oceancdCluster")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def delete_oceancd_cluster(self, cluster_id: str):
        """
        Delete an existing Ocean CD cluster.

        # Arguments
        cluster_id (String): OceanCD Cluster ID

        # Returns
        (Boolean): Response Status
        """

        return self.send_delete(
            url=self.__base_oceancd_cluster_url + "/" + cluster_id,
            entity_name="oceancdCluster")

    def create_oceancd_verification_provider(self,
                                             verification_provider: oceancd_vp.VerificationProvider):
        """
        Create Ocean CD Verification Provider. only one provider type can be defined

        # Arguments
        verification_provider (VerificationProvider): OceanCD Verification Provider

        # Returns
        (Object) : OceanCD Verification Provider
        """
        request = oceancd_vp.CreateVerificationProviderRequest(verification_provider)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_oceancd_vp_url,
            entity_name="oceancdVerificationProvider")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_all_oceancd_verification_providers(self):
        """
        List all Ocean CD verification providers.

        # Returns
        (List): List of Ocean CD Verification Provider API response
        """
        response = self.send_get(
            url=self.__base_oceancd_vp_url,
            entity_name="oceancdVerificationProvider")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_oceancd_verification_provider(self, name: str):
        """
        Get an existing  Ocean CD Verification Provider.

        # Arguments
        name (String): The identifier of the Ocean CD Verification Provider

        # Returns
        (Object): OceanCD Verification Provider API response
        """
        response = self.send_get(
            url=self.__base_oceancd_vp_url + "/" + name,
            entity_name="oceancdVerificationProvider")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_oceancd_verifcation_provider(self, provider_update: oceancd_vp.VerificationProvider,
                                            name: str):
        """
        Full Update of Ocean CD Verification Provider configuration. All non included fields will be nullified

        # Arguments
        name (String): The identifier name of the Ocean CD Verification Provider
        provider_update (VerificationProvider): VerificationProvider Object

        # Returns
        (Boolean): Response Status
        """
        request = oceancd_vp.CreateVerificationProviderRequest(provider_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_oceancd_vp_url + "/" + name,
            entity_name="oceancdVerificationProvider")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def delete_oceancd_verification_provider(self, name: str):
        """
        Delete an existing Ocean CD Verification Provider.

        # Arguments
        name (String): OceanCD Cluster ID

        # Returns
        (Boolean): Response Status
        """

        return self.send_delete(
            url=self.__base_oceancd_vp_url + "/" + name,
            entity_name="oceancdVerificationProvider")

    def create_oceancd_verification_template(self,
                                             verification_template: oceancd_vt.VerificationTemplate):
        """
        Create Ocean CD Verification Template.

        # Arguments
        verification_template (VerificationTemplate) : OceanCD Verification Template

        # Returns
        (Object) : OceanCD Verification Template
        """
        request = oceancd_vt.CreateVerificationTemplateRequest(verification_template)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_oceancd_vt_url,
            entity_name="oceancdVerificationTemplate")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_oceancd_verification_templates(self):
        """
        List all Ocean CD verification templates.

        # Returns
        (List): List of Ocean CD Verification Template API response
        """
        response = self.send_get(
            url=self.__base_oceancd_vt_url,
            entity_name="oceancdVerificationTemplate")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_oceancd_verification_template(self, name: str):
        """
        Get an existing Ocean CD Verification Template.

        # Arguments
        name (String): The identifier of the Ocean CD Verification Template

        # Returns
        (Object): OceanCD Verification Template API response
        """
        response = self.send_get(
            url=self.__base_oceancd_vt_url + "/" + name,
            entity_name="oceancdVerificationTemplate")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_oceancd_verifcation_template(self, template_update: oceancd_vt.VerificationTemplate, name: str):
        """
        Full Update of Ocean CD Verification Template configuration. All non included fields will be nullified

        # Arguments
        name (String): The identifier name of the Ocean CD Verification Template
        template_update (VerificationTemplate): VerificationTemplate Object

        # Returns
        (Boolean): Response Status
        """
        request = oceancd_vt.CreateVerificationTemplateRequest(template_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_oceancd_vt_url + "/" + name,
            entity_name="oceancdVerificationTemplate")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def delete_oceancd_verification_template(self, name: str):
        """
        Delete an existing Ocean CD Verification Template.

        # Arguments
        name (String): The identifier name of the Ocean CD Verification Template

        # Returns
        (Boolean): Response Status
        """

        return self.send_delete(
            url=self.__base_oceancd_vt_url + "/" + name,
            entity_name="oceancdVerificationTemplate")

    def create_oceancd_strategy(self, strat: strategy.Strategy):
        """
        Create Ocean CD Strategy.

        # Arguments
        strat (Strategy) : OceanCD Strategy

        # Returns
        (Object) : OceanCD Strategy
        """
        request = strategy.CreateStrategyRequest(strat)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_strategy_url,
            entity_name="oceancdStrategy")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_oceancd_strategy(self):
        """
        List all Ocean CD strategy.

        # Returns
        (List): List of Ocean CD Strategy API response
        """
        response = self.send_get(
            url=self.__base_strategy_url,
            entity_name="oceancdStrategy")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_oceancd_strategy(self, name: str):
        """
        Get an existing Ocean CD Strategy.

        # Arguments
        name (String): The identifier of the Ocean CD Strategy

        # Returns
        (Object): OceanCD Strategy API response
        """
        response = self.send_get(
            url=self.__base_strategy_url + "/" + name,
            entity_name="oceancdStrategy")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_oceancd_strategy(self, strategy_update: strategy.Strategy, name: str):
        """
        Full Update of Ocean CD strategy configuration. All non included fields will be nullified

        # Arguments
        name (String): The identifier name of the Ocean CD Strategy
        strategy_update (Strategy): Strategy Object

        # Returns
        (Boolean): Response Status
        """
        request = strategy.CreateStrategyRequest(strategy_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_strategy_url + "/" + name,
            entity_name="oceancdStrategy")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def delete_oceancd_strategy(self, name: str):
        """
        Delete an existing Ocean CD Strategy.

        # Arguments
        name (String): OceanCD Strategy

        # Returns
        (Boolean): Response Status
        """

        return self.send_delete(
            url=self.__base_strategy_url + "/" + name,
            entity_name="oceancdStrategy")

    def create_oceancd_rollout_spec(self, rollouts: rollout_spec.RolloutSpec):
        """
        Create Ocean CD rollout spec.

        # Arguments
        rollouts (RolloutSpec): OceanCD Rollout Spec

        # Returns
        (Object): OceanCD Rollout Spec
        """
        request = rollout_spec.CreateRolloutSpecRequest(rollouts)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_rollout_spec_url,
            entity_name="oceancdRolloutSpec")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_oceancd_rollout_specs(self):
        """
        List all Ocean CD rollout specs.

        # Returns
        (List): List of Ocean CD Rollout Spec API response
        """
        response = self.send_get(
            url=self.__base_rollout_spec_url,
            entity_name="oceancdRolloutSpec")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_oceancd_rollout_spec(self, name: str):
        """
        Get the configuration of an existing Ocean CD rollout spec.

        # Arguments
        name (String): The identifier of the Ocean CD Rollout Spec

        # Returns
        (Object): OceanCD Rollout Spec API response
        """
        response = self.send_get(
            url=self.__base_rollout_spec_url + "/" + name,
            entity_name="oceancdRolloutSpec")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_oceancd_rollout_spec(self, rollout_spec_update: rollout_spec.RolloutSpec, name: str):
        """
        Full Update Ocean CD rollout spec configuration. All non included fields will be nullified

        # Arguments
        name (String): The identifier name of the Ocean CD Rollout Spec
        rollout_spec_update (RolloutSpec): Rollout Spec Object

        # Returns
        (Boolean): Response Status
        """
        request = rollout_spec.CreateRolloutSpecRequest(rollout_spec_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_rollout_spec_url + "/" + name,
            entity_name="oceancdRolloutSpec")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def delete_oceancd_rollout(self, name: str):
        """
        Delete an existing Ocean CD Rollout Spec.

        # Arguments
        name (String): OceanCD Rollout Spec

        # Returns
        (Boolean): Response Status
        """

        return self.send_delete(
            url=self.__base_rollout_spec_url + "/" + name,
            entity_name="oceancdRolloutSpec")

    def describe_rollout_by_id(self, rollout_id: str):
        """
        Get Ocean CD single rollout

        # Arguments
        rollout_id (String): The identifier of the Ocean CD rollout

        # Returns
        (Object): OceanCD Rollout API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + "/" + rollout_id,
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_rollout_actions(self, rollout_id: str, action_update: rollout.Action):
        """
        Execute action on an existing Ocean CD rollout.

        # Arguments
        rollout_id (String): The identifier of the Ocean CD rollout

        # Returns
        (Boolean): Response Status
        """
        request = rollout.CreateRolloutActionRequest(action_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_rollout_url + "/" + rollout_id,
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def list_ocean_cd_rollouts(self, from_date: str):
        """
        List Ocean CD rollouts.

        # Arguments
        from_date (String): From Date

        # Returns
        (Object): OceanCD Rollout List API response
        """
        query_params = dict(fromDate=from_date)

        response = self.send_get(
            url=self.__base_rollout_url,
            entity_name="oceancdRollout",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_describe_rollout_status(self, rollout_id: str):
        """
        Ocean CD rollout status.

        # Arguments
        rollout_id (String): The identifier of the Ocean CD rollout

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/'+rollout_id+'/status',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_latest_rollouts(self, cluster_id: str, count: str, namespace: str, spot_deployment: str):
        """
        Ocean CD Latest rollout/s.

        # Arguments
        clusterId (String): clusterId configured.
        count (String): How many responses intended, sorted from the last one down.
        namespace (String): namespace name configured
        spot_deployment (String): SpotDeployment name configured

        # Returns
        (Object): OceanCD Rollout List API response
        """
        query_params = dict(cluster_id=cluster_id, count=count, namespace=namespace, spot_deployment=spot_deployment)

        response = self.send_get(
            url=self.__base_rollout_url + '/latest',
            entity_name="oceancdRollout",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_describe_ongoing_rollouts(self):
        """
        Ocean CD Ongoing rollouts.

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/ongoing',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_describe_rollout_verification(self, rollout_id: str):
        """
        Ocean CD rollout verification.

        # Arguments
        rolloutId (String): The identifier of the Ocean CD Verification Response

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/'+rollout_id+'/verification',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_rollout_definition(self, rollout_id: str):
        """
        Ocean CD rollout definition.

        # Arguments
        rolloutId (String): The identifier of the Ocean CD Definition Response

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/'+rollout_id+'/definition',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_rollout_resource(self, rollout_id: str):
        """
        Ocean CD Resource

        # Arguments
        rolloutId (String): The identifier of the Ocean CD Resource Response

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/'+rollout_id+'/resource',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_rollout_phase(self, rollout_id: str):
        """
        Ocean CD Rollout phase.

        # Arguments
        rolloutId (String): The identifier of the Ocean CD phase

        # Returns
        (Object): OceanCD Rollout List API response
        """
        response = self.send_get(
            url=self.__base_rollout_url + '/'+rollout_id+'/phase',
            entity_name="oceancdRollout")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_all_ocean_cd_workloads(self):
        """
        List all Ocean CD workloads.

        # Returns
        (Object): OceanCD Workloads API response
        """
        response = self.send_get(
            url=self.__base_workload_url,
            entity_name="oceancdWorkloads")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_describe_workloads_migration_process(self, deployment_name: str, namespace: str, cluster_id: str):
        """
        Describe How To Migrate Deployment CRD To SpotDeployment DRD.

        # Arguments
        deployment_name (String): The identifier name of the Ocean CD Deployment
        namespace (String): The workload's namespace
        cluster_id (String): Cluster id where the workload is running

        # Returns
        (Object): OceanCD Workloads Migration API response
        """
        query_params = dict(cluster_id=cluster_id)

        response = self.send_get(
            url=self.__base_workload_url + '/namespace/'+namespace+'/deployment/'+deployment_name+'/migrate',
            entity_name="oceancdWorkloads",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_workloads_active_operations(self, workload_id: str):
        """
        Describe What Operation Are Active For Specific Workload.

        # Arguments
        workload_id (String): The workload's Id

        # Returns
        (Object): OceanCD Workloads Active Operations API response
        """
        response = self.send_get(
            url=self.__base_workload_url + '/'+workload_id+'/activeOperation',
            entity_name="oceancdWorkloads")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_describe_workloads_revision(self, workload_id: str, namespace: str, cluster_id: str, kind: str):
        """
        Describe Workload's Revision.

        # Arguments
        workload_id (String): The workload's Id
        namespace (String): Workload's namespace name
        cluster_id (String): Cluster id where the workload is running
        kind (String): Kind of workload, currently only SpotDeployment is supported

        # Returns
        (Object): OceanCD Workloads Revision API response
        """

        query_params = dict(clusterId=cluster_id, kind=kind)
        response = self.send_get(
            url=self.__base_workload_url + '/'+workload_id+'/namespace/'+namespace+'/revision',
            entity_name="oceancdWorkloads",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def ocean_cd_describe_workloads_graph(self, namespace: str, workload_name: str, cluster_id: str, kind: str):
        """
        Ocean CD workload graph.

        # Arguments
        namespace (String): Workload's namespace name
        workload_name (String): Workload name
        cluster_id (String): Cluster id where the workload is running
        kind (String): Kind of workload, currently only SpotDeployment is supported

        # Returns
        (Object): OceanCD Workloads Revision API response
        """
        query_params = dict(cluster_id=cluster_id, kind=kind)
        response = self.send_get(
            url=self.__base_workload_url + '/'+workload_name+'/namespace/'+namespace+'/graph',
            entity_name="oceancdWorkloads",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def ocean_cd_restart_workload(self, workload_id: str):
        """
        Restart Workload By WorkloadId.

        # Arguments
        workload_id (String): The workload's Id

        # Returns
        (Boolean): Response Status
        """
        response = self.send_put(
            url=self.__base_workload_url + '/'+workload_id+'/restart',
            entity_name="oceancdWorkloads")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def ocean_cd_retry_workload(self, revision_id: str, workload_id: str, rollout_id: str):
        """
        Retry Workload By WorkloadId,RevisionId And RolloutId

        # Arguments
        revision_id (String): Relevant Revision Id
        workload_id (String): The workload's Id
        rollout_id (String): The rollout Id

        # Returns
        (Boolean): Response Status
        """
        query_params = dict(rollout_id=rollout_id)
        response = self.send_put(
            url=self.__base_workload_url + '/'+workload_id+'/revision/'+revision_id+'/retry',
            entity_name="oceancdWorkloads",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def ocean_cd_rollback_workload(self, revision_id: str, workload_id: str, rollout_id: str):
        """
        Rollback Workload By WorkloadId,RevisionId And RolloutId

        # Arguments
        revision_id (String): Relevant Revision Id
        workload_id (String): The workload's Id
        rollout_id (String): The rollout Id

        # Returns
        (Boolean): Response Status
        """
        query_params = dict(rollout_id=rollout_id)
        response = self.send_put(
            url=self.__base_workload_url + '/'+workload_id+'/revision/'+revision_id+'/rollback',
            entity_name="oceancdWorkloads",
            query_params=query_params)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]



# Always prefer setuptools over distutils
from codecs import open
from os import path

import sys

from setuptools import setup

needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

here = path.abspath(path.dirname(__file__))

# Get the long description from the README.md file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

version = {}
with open("spotinst_sdk2/version.py") as fp:
    exec(fp.read(), version)

setup(
    name='spotinst-sdk2',

    version=version['__version__'],
    # start from Python 3.6
    python_requires='>=3.6.0',
    description='A Python SDK for Spotinst',
    long_description=long_description,
    long_description_content_type='text/markdown',

    # The project's main homepage.
    url='https://github.com/spotinst/spotinst-sdk-python',

    # Author details
    author='Spotinst',
    author_email='service@spotinst.com',

    # Choose your license
    license='MIT',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],

    keywords='spotinst spot instances aws azure ec2 cloud infrastructure development elastigroup',
    packages=[
    "spotinst_sdk2", 
        "spotinst_sdk2.clients", 
            "spotinst_sdk2.clients.admin",
            "spotinst_sdk2.clients.elastigroup",
            "spotinst_sdk2.clients.functions",
            "spotinst_sdk2.clients.hpc",
            "spotinst_sdk2.clients.mcs",
            "spotinst_sdk2.clients.mlb",
            "spotinst_sdk2.clients.mrscaler",
            "spotinst_sdk2.clients.ocean",
            "spotinst_sdk2.clients.ocean_cd",
            "spotinst_sdk2.clients.subscription",
            "spotinst_sdk2.clients.setup",
            "spotinst_sdk2.clients.managed_instance",
            "spotinst_sdk2.clients.stateful_node",
        "spotinst_sdk2.models",
            "spotinst_sdk2.models.admin",
            "spotinst_sdk2.models.elastigroup",
                "spotinst_sdk2.models.elastigroup.aws",
                "spotinst_sdk2.models.elastigroup.azure",
                "spotinst_sdk2.models.elastigroup.azure_v3",
                "spotinst_sdk2.models.elastigroup.gcp",
            "spotinst_sdk2.models.functions",
            "spotinst_sdk2.models.hpc",
                "spotinst_sdk2.models.hpc.aws",
            "spotinst_sdk2.models.mlb",
            "spotinst_sdk2.models.mrscaler",
                "spotinst_sdk2.models.mrscaler.aws",
            "spotinst_sdk2.models.ocean",
                "spotinst_sdk2.models.ocean.aws",
                "spotinst_sdk2.models.ocean.azure",
            "spotinst_sdk2.models.ocean_cd",
                "spotinst_sdk2.models.ocean_cd",
            "spotinst_sdk2.models.setup",
                "spotinst_sdk2.models.setup.azure",
                "spotinst_sdk2.models.setup.gcp",
            "spotinst_sdk2.models.subscription",
            "spotinst_sdk2.models.managed_instance",
                "spotinst_sdk2.models.managed_instance.aws",
            "spotinst_sdk2.models.stateful_node"
    ],
    install_requires=['requests', 'PyYaml'],

    setup_requires=[] + pytest_runner,
    tests_require=["pytest"]
)



import json

none = "d3043820717d74d9a17694c176d39733"


# region Elastigroup
class Elastigroup:
    """
    # Arguments
    name: str
    region: str
    resource_group_name: str
    description: str
    capacity: Capacity
    strategy: Strategy
    compute: Compute
    scaling: Scaling
    health: Health
    scheduling: Scheduling
    """

    def __init__(
            self,
            name=none,
            region=none,
            resource_group_name=none,
            description=none,
            capacity=none,
            strategy=none,
            compute=none,
            scaling=none,
            health=none,
            scheduling=none):
        self.name = name
        self.region = region
        self.resource_group_name = resource_group_name
        self.description = description
        self.capacity = capacity
        self.strategy = strategy
        self.compute = compute
        self.scaling = scaling
        self.health = health
        self.scheduling = scheduling


# endregion

# region Capacity
class Capacity:
    """
    # Arguments
    minimum: int
    maximum: int
    target: int
    """

    def __init__(
            self,
            minimum=none,
            maximum=none,
            target=none):
        self.minimum = minimum
        self.maximum = maximum
        self.target = target


# endregion

# region Compute
class Compute:
    """
    # Arguments
    launch_specification: LaunchSpecification
    os: str
    preferred_zones: List[str]
    vm_sizes: VmSizes
    zones: List[str]

    """

    def __init__(
            self,
            launch_specification=none,
            os=none,
            preferred_zones=none,
            vm_sizes=none,
            zones=none,
    ):
        self.launch_specification = launch_specification
        self.os = os
        self.preferred_zones = preferred_zones
        self.vm_sizes = vm_sizes
        self.zones = zones


class LaunchSpecification:
    """
    # Arguments
    boot_diagnostics: BootDiagnostics
    custom_data: str
    data_disks: List[DataDisk]
    extensions: list[Extension]
    image: Image
    load_balancers_config: LoadBalancerConfig
    login: Login
    managed_service_identities: list[ManagedServiceIdentity]
    network: Network
    os_disk: OsDisk
    secrets: List[Secret]
    shutdown_script: str
    tags: list[Tag]
    vm_name_prefix: str
    """

    def __init__(
            self,
            boot_diagnostics=none,
            custom_data=none,
            data_disks=none,
            extensions=none,
            image=none,
            load_balancers_config=none,
            login=none,
            managed_service_identities=none,
            network=none,
            os_disk=none,
            secrets=none,
            shutdown_script=none,
            tags=none,
            vm_name_prefix=none):
        self.boot_diagnostics = boot_diagnostics
        self.custom_data = custom_data
        self.data_disks = data_disks
        self.extensions = extensions
        self.image = image
        self.load_balancers_config = load_balancers_config
        self.login = login
        self.managed_service_identities = managed_service_identities
        self.network = network
        self.os_disk = os_disk
        self.secrets = secrets
        self.shutdown_script = shutdown_script
        self.tags = tags
        self.vm_name_prefix = vm_name_prefix


class BootDiagnostics:
    """
    # Arguments
    is_enabled: bool
    storage_uri = str
    type: str
    """

    def __init__(
            self,
            is_enabled=none,
            storage_uri=none,
            type=none):
        self.is_enabled = is_enabled
        self.storage_uri = storage_uri
        self.type = type


class DataDisk:
    """
    # Arguments
    lun: int
    size_gb = int
    type: str
    """

    def __init__(
            self,
            lun=none,
            size_g_b=none,
            type=none):
        self.lun = lun
        self.size_g_b = size_g_b
        self.type = type


class Extension:
    """
    # Arguments
    api_version: str
    auto_upgrade_minor_version: bool
    minor_version_auto_upgrade: bool
    name: str
    publisher: str
    type: str
    """

    def __init__(
            self,
            api_version=none,
            auto_upgrade_minor_version=none,
            minor_version_auto_upgrade=none,
            name=none,
            publisher=none,
            type=none):
        self.api_version = api_version
        self.auto_upgrade_minor_version = auto_upgrade_minor_version
        self.minor_version_auto_upgrade = minor_version_auto_upgrade
        self.name = name
        self.publisher = publisher
        self.type = type


class Image:
    """
    # Arguments
    marketplace: Marketplace
    custom: Custom
    gallery: Gallery
    """

    def __init__(self, marketplace=none, custom=none, gallery=none):
        self.marketplace = marketplace
        self.custom = custom
        self.gallery = gallery


class Marketplace:
    """
    # Arguments
    publisher: str
    offer: str
    sku: str
    version: str
    """

    def __init__(
            self,
            publisher=none,
            offer=none,
            sku=none,
            version=none):
        self.publisher = publisher
        self.offer = offer
        self.sku = sku
        self.version = version


class Custom:
    """
    # Arguments
    resource_group_name: str
    name: str
    """

    def __init__(
            self,
            resource_group_name=none,
            name=none):
        self.resource_group_name = resource_group_name
        self.name = name


class Gallery:
    """
    # Arguments
    gallery_name: str
    image_name: str
    resource_group_name: str
    spot_account_id: str
    version: str
    """

    def __init__(
            self,
            gallery_name=none,
            image_name=none,
            resource_group_name=none,
            spot_account_id=none,
            version=none):
        self.gallery_name = gallery_name
        self.image_name = image_name
        self.resource_group_name = resource_group_name
        self.spot_account_id = spot_account_id
        self.version = version


class LoadBalancerConfig:
    """
    # Arguments
    load_balancers: list[LoadBalancer]
    """

    def __init__(self, load_balancers=none):
        self.load_balancers = load_balancers


class LoadBalancer:
    """
    # Arguments
    balancer_id: str
    target_set_id: str
    auto_weight: bool
    type: str
    resource_group_name: str
    application_gateway_name: str
    backend_pool_name: str
    """

    def __init__(
            self,
            balancer_id=none,
            target_set_id=none,
            auto_weight=none,
            type=none,
            resource_group_name=none,
            application_gateway_name=none,
            backend_pool_name=none):
        self.balancer_id = balancer_id
        self.target_set_id = target_set_id
        self.auto_weight = auto_weight
        self.type = type
        self.resource_group_name = resource_group_name
        self.application_gateway_name = application_gateway_name
        self.backend_pool_name = backend_pool_name


class Login:
    """
    # Arguments
    ssh_public_key: str
    user_name: str
    password: str
    """

    def __init__(
            self,
            ssh_public_key=none,
            user_name=none,
            password=none):
        self.ssh_public_key = ssh_public_key
        self.user_name = user_name
        self.password = password


class ManagedServiceIdentity:
    """
    # Arguments
    resource_group_name: str
    name: str
    """

    def __init__(
            self,
            resource_group_name=none,
            name=none):
        self.resource_group_name = resource_group_name
        self.name = name


class Network:
    """
    # Arguments
    network_interfaces: List[NetworkInterface]
    resource_group_name: str
    virtual_network_name: str
    """

    def __init__(
            self,
            network_interfaces=none,
            virtual_network_name=none,
            resource_group_name=none):
        self.network_interfaces = network_interfaces
        self.virtual_network_name = virtual_network_name
        self.resource_group_name = resource_group_name


class NetworkInterface:
    """
    # Arguments
    additional_ip_configurations: List[AdditionalIpConfiguration]
    application_security_groups: List[ApplicationSecurityGroup]
    assign_public_ip: bool
    enable_ip_forwarding: bool
    is_primary: bool
    private_ip_addresses: List[str]
    public_ips: List[PublicIp]
    public_ip_sku: str
    security_group: SecurityGroup
    subnet_name: str
    """

    def __init__(
            self,
            additional_ip_configurations=none,
            application_security_groups=none,
            assign_public_ip=none,
            enable_ip_forwarding=none,
            is_primary=none,
            private_ip_addresses=none,
            public_ips=none,
            public_ip_sku=none,
            security_group=none,
            subnet_name=none):
        self.additional_ip_configurations = additional_ip_configurations
        self.application_security_groups = application_security_groups
        self.assign_public_ip = assign_public_ip
        self.enable_ip_forwarding = enable_ip_forwarding
        self.is_primary = is_primary
        self.private_ip_addresses = private_ip_addresses
        self.public_ips = public_ips
        self.public_ip_sku = public_ip_sku
        self.security_group = security_group
        self.subnet_name = subnet_name


class AdditionalIpConfiguration:
    """
    # Arguments
    name: str
    private_ip_address_version: str
    """

    def __init__(
            self,
            name=none,
            private_ip_address_version=none):
        self.name = name
        self.private_ip_address_version = private_ip_address_version


class ApplicationSecurityGroup:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name=none,
            resource_group_name=none):
        self.name = name
        self.resource_group_name = resource_group_name


class PublicIp:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name=none,
            resource_group_name=none):
        self.name = name
        self.resource_group_name = resource_group_name


class SecurityGroup:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name=none,
            resource_group_name=none):
        self.name = name
        self.resource_group_name = resource_group_name


class OsDisk:
    """
    # Arguments
    size: int
    type: str
    """

    def __init__(
            self,
            size=none,
            type=none):
        self.size = size
        self.type = type


class Secret:
    """
    # Arguments
    source_vault: SourceVault
    vault_certificates: List[VaultCertificate]
    """

    def __init__(
            self,
            source_vault=none,
            vault_certificates=none):
        self.source_vault = source_vault
        self.vault_certificates = vault_certificates


class SourceVault:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name=none,
            resource_group_name=none):
        self.name = name
        self.resource_group_name = resource_group_name


class VaultCertificate:
    """
    # Arguments
    certificate_store: str
    certificate_url: str
    """

    def __init__(
            self,
            certificate_store=none,
            certificate_url=none):
        self.certificate_store = certificate_store
        self.certificate_url = certificate_url


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """

    def __init__(
            self,
            tag_key=none,
            tag_value=none):
        self.tag_key = tag_key
        self.tag_value = tag_value


class VmSizes:
    """
    # Arguments
    od_sizes: list[str]
    preferred_spot_sizes: list[str]
    spot_sizes: List[str]
    """

    def __init__(
            self,
            od_sizes=none,
            preferred_spot_sizes=none,
            spot_sizes=none):
        self.od_sizes = od_sizes
        self.preferred_spot_sizes = preferred_spot_sizes
        self.spot_sizes = spot_sizes


# endregion

# region Health
class Health:
    """
    # Arguments
    health_check_types: list[str]
    auto_healing: bool
    grace_period: int
    unhealthy_duration: int
    """

    def __init__(
            self,
            health_check_types=none,
            auto_healing=none,
            grace_period=none,
            unhealthy_duration=none):
        self.health_check_types = health_check_types
        self.auto_healing = auto_healing
        self.grace_period = grace_period
        self.unhealthy_duration = unhealthy_duration


# endregion

# region Scaling
class Scaling:
    """
    # Arguments
    up:  list[ScalingPolicy]
    down: list[ScalingPolicy]
    """

    def __init__(self, up=none, down=none):
        self.up = up
        self.down = down


class ScalingPolicy:
    """
    # Arguments
    is_enabled: bool
    policy_name: str
    namespace: str
    metric_name: str
    dimensions: list[ScalingPolicyDimension]
    statistic: str
    unit: str
    threshold: float
    period: int
    evaluation_periods: int
    cooldown: int
    action: ScalingPolicyAction
    operator: str
    """

    def __init__(
            self,
            is_enabled=none,
            policy_name=none,
            namespace=none,
            metric_name=none,
            dimensions=none,
            statistic=none,
            unit=none,
            threshold=none,
            period=none,
            evaluation_periods=none,
            cooldown=none,
            action=none,
            operator=none):
        self.is_enabled = is_enabled
        self.policy_name = policy_name
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.period = period
        self.evaluation_periods = evaluation_periods
        self.cooldown = cooldown
        self.action = action
        self.operator = operator


class ScalingPolicyDimension:
    """
    # Arguments
    key: str
    value: str
    """

    def __init__(self, key=none, value=none):
        self.key = key
        self.value = value


class ScalingPolicyAction:
    """
    # Arguments
    type: str
    adjustment: str
    target: int
    maximum: int
    minimum: int
    """

    def __init__(
            self,
            type=none,
            adjustment=none,
            target=none,
            maximum=none,
            minimum=none):
        self.type = type
        self.adjustment = adjustment
        self.target = target
        self.maximum = maximum
        self.minimum = minimum


# endregion

# region Strategy
class Strategy:
    """
    # Arguments
    draining_timeout: int
    on_demand_count: int
    optimization_windows: List[str]
    orientation: str
    revert_to_spot: RevertToSpot
    signals: List[Signal]
    spot_percentage: int
    fallback_to_od: bool
    """

    def __init__(
            self,
            draining_timeout=none,
            on_demand_count=none,
            optimization_windows=none,
            orientation=none,
            revert_to_spot=none,
            signals=none,
            spot_percentage=none,
            fallback_to_od=none):
        self.draining_timeout = draining_timeout
        self.on_demand_count = on_demand_count
        self.optimization_windows = optimization_windows
        self.orientation = orientation
        self.revert_to_spot = revert_to_spot
        self.signals = signals
        self.spot_percentage = spot_percentage
        self.fallback_to_od = fallback_to_od


class RevertToSpot:
    """
    # Arguments
    perform_at: str
    """

    def __init__(self, perform_at=none):
        self.perform_at = perform_at


class Signal:
    """
    # Arguments
    timeout: str
    type: str
    """

    def __init__(self, timeout=none, type=none):
        self.timeout = timeout
        self.type = type


# endregion

# region Scheduling
class Scheduling:
    """
    # Arguments
    tasks: list[SchedulingTask]
    """

    def __init__(self, tasks=none):
        self.tasks = tasks


class SchedulingTask:
    """
    # Arguments
    is_enabled: bool
    frequency: str
    start_time: str
    cron_expression: str
    type: str
    scale_target_capacity: int
    scale_min_capacity: int
    scale_max_capacity: int
    batch_size_percentage: int
    grace_period: int
    adjustment: int
    adjustment_percentage: int
    target_capacity: int
    min_capacity: int
    max_capacity: int
    """

    def __init__(
            self,
            is_enabled=none,
            frequency=none,
            start_time=none,
            cron_expression=none,
            type=none,
            scale_target_capacity=none,
            scale_min_capacity=none,
            scale_max_capacity=none,
            batch_size_percentage=none,
            grace_period=none,
            adjustment=none,
            adjustment_percentage=none,
            target_capacity=none,
            min_capacity=none,
            max_capacity=none):
        self.is_enabled = is_enabled
        self.frequency = frequency
        self.start_time = start_time
        self.cron_expression = cron_expression
        self.type = type
        self.scale_target_capacity = scale_target_capacity
        self.scale_min_capacity = scale_min_capacity
        self.scale_max_capacity = scale_max_capacity
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.adjustment = adjustment
        self.adjustment_percentage = adjustment_percentage
        self.target_capacity = target_capacity
        self.min_capacity = min_capacity
        self.max_capacity = max_capacity


# endregion

# region Requests

class DetachConfiguration:
    """
    # Arguments
    draining_timeout: int
    should_decrement_target_capacity: bool
    should_terminate_vms: bool
    instances_to_detach: list[str]
    """
    def __init__(
            self,
            draining_timeout=none,
            should_decrement_target_capacity=none,
            should_terminate_vms=none,
            vms_to_detach=none):
        self.draining_timeout = draining_timeout
        self.should_decrement_target_capacity = should_decrement_target_capacity
        self.should_terminate_vms = should_terminate_vms
        self.vms_to_detach = vms_to_detach

class ElastigroupDetachVMsRequest:
    def __init__(self, detach_configuration):
        self.should_decrement_target_capacity = detach_configuration.should_decrement_target_capacity
        self.draining_timeout = detach_configuration.draining_timeout
        self.vms_to_detach = detach_configuration.vms_to_detach
        self.should_terminate_vms = detach_configuration.should_terminate_vms

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)

class DeploymentConfiguration:
    """
    # Arguments
    batch_min_healthy_percentage: int
    batch_size_percentage: int
    draining_timeout: int
    grace_period: int
    health_check_types: list[str]
    """

    def __init__(
            self,
            batch_min_healthy_percentage=none,
            batch_size_percentage=none,
            draining_timeout=none,
            grace_period=none,
            health_check_types=none):
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.batch_size_percentage = batch_size_percentage
        self.draining_timeout = draining_timeout
        self.grace_period = grace_period
        self.health_check_types = health_check_types

class ElastigroupDeploymentRequest:
    def __init__(self, deployment_configuration):
        self.deployment = deployment_configuration

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

class Process:
    """
    # Arguments
    name: str
    ttl_in_minutes: int
    """
    def __init__(
            self,
            name=none,
            ttl_in_minutes=none):
        self.name = name
        self.ttl_in_minutes = ttl_in_minutes

class ElastigroupProcessesRequest:
    def __init__(self, processes):
        self.processes = processes

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

class ElastigroupCreateRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupUpdateRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupUpdateCapacityRequest:
    def __init__(self, capacity):
        self.capacity = capacity

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion



import json

none = "d3043820717d74d9a17694c176d39733"

# region ASG
class ASG:
	"""
	# Arguments
	product: str
	spot_instance_types: List[str]
	name: str
	"""
	def __init__(
		self,
		product=none,
		spot_instance_types=none,
		name=none):

		self.product = product
		self.spot_instance_types = spot_instance_types
		self.name = name

# endregion


class ImportASGRequest:
    def __init__(self, group):
        self.group = group

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)




import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.mlb as spotinst_mlb

class MlbClient(Client):
    __base_lb_url = "/loadBalancer"

    # region MLB
    def get_all_mlb_runtime(self):
        """
        Get all MLB runtime 
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_get(
            url=self.__base_lb_url +
            "/runtime",
            entity_name="mlb runtime"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal     

    def get_mlb_runtime(self, runtime_id):
        """
        Get MLB runtime
        
        # Arguments
        runtime_id (String): Runtime id name
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_get(
            url=self.__base_lb_url +
            "/runtime/" + runtime_id,
            entity_name="mlb runtime"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal     

    def deregister_mlb_runtime(self, runtime_id):
        """
        Deregister MLB runtime
        
        # Arguments
        runtime_id (String): Runtime id name
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_put(
            url=self.__base_lb_url +
            "/runtime/" + runtime_id +
            "/deregister",
            entity_name="mlb runtime"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal   

    def delete_mlb_runtime(self, runtime_id):
        """
        Delete MLB runtime
        
        # Arguments
        runtime_id (String): Runtime id name
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_delete(
            url=self.__base_lb_url +
            "/runtime/" + runtime_id,
            entity_name="mlb runtime"
        )

        return response 

    def create_mlb_deployment(self, deployment_name):
        """
        Create MLB deployment
        
        # Arguments
        deployment_name (String): Deployment name
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_post(
            url=self.__base_lb_url +
            "/deployment",
            body=json.dumps(dict(deployment=dict(name=deployment_name))),
            entity_name="mlb deployment"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_deployment(self, deployment_id, deployment_name):
        """
        Update MLB deployment
        
        # Arguments
        deployment_name (String): Deployment name
        deployment_id (String): Deployment Id
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_put(
            url=self.__base_lb_url +
            "/deployment/" + deployment_id,
            body=json.dumps(dict(deployment=dict(name=deployment_name))),
            entity_name="mlb deployment"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)
        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_deployment(self, deployment_id):
        """
        Get MLB deployment
        
        # Arguments
        deployment_id (String): Deployment Id
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_get(
            url=self.__base_lb_url +
            "/deployment/"+deployment_id,
            entity_name="mlb deployment"
        )
        
        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_deployment(self):
        """
        Get All MLB deployment
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_get(
            url=self.__base_lb_url +
            "/deployment",
            entity_name="mlb deployment"
        )
        
        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_deployment(self, deployment_id):
        """
        Delete MLB deployment
        
        # Arguments
        deployment_id (String): Deployment Id
        
        # Returns
        (Object): Spotinst API response 
        """ 
        response = self.send_delete(
            url=self.__base_lb_url +
            "/deployment/"+deployment_id,
            entity_name="mlb deployment"
        )

        return response

    def create_mlb_balancer(self, balancer):
        """
        Create MLB balancer
        
        # Arguments
        balancer (Balancer): Balancer Object
        
        # Returns
        (Object): Spotinst API response 
        """ 
        balancer = spotinst_mlb.BalancerRequest(balancer)

        excluded_group_dict = self.exclude_missing(json.loads(balancer.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/balancer",
            body=body_json,
            entity_name="mlb balancer"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal        

    def update_mlb_balancer(self, balancer_id, balancer):
        """
        Create MLB balancer
        
        # Arguments
        balancer_id (String): Balancer Id
        balancer (Balancer): Balancer Object
        
        # Returns
        (Object): Spotinst API response 
        """ 
        balancer = spotinst_mlb.BalancerRequest(balancer)

        excluded_group_dict = self.exclude_missing(json.loads(balancer.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/balancer/" + balancer_id,
            body=body_json,
            entity_name="mlb balancer"
        )
        
        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_balancer(self, balancer_id):
        """
        Get MLB balancer
        
        # Arguments
        balancer_id (String): Balancer Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/balancer/" + balancer_id,
            entity_name="mlb balancer" 
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal        

    def get_all_mlb_balancer(self):
        """
        Get All MLB balancer
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/balancer/",
            entity_name="mlb balancer" 
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal   

    def delete_mlb_balancer(self, balancer_id):
        """
        Delete MLB balancer
        
        # Arguments
        balancer_id (String): Balancer Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/balancer/" + balancer_id,
            entity_name="mlb balancer" 
        )

        return response   

    def create_mlb_target_set(self, target_set):
        """
        Create MLB target set
        
        # Arguments
        target_set (TargetSet): TargetSet Object
        
        # Returns
        (Object): Spotinst API response 
        """
        target_set = spotinst_mlb.TargetSetRequest(target_set)

        excluded_group_dict = self.exclude_missing(json.loads(target_set.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/targetSet",
            body=body_json,
            entity_name="mlb target set"
        )
        
        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_target_set(self, target_set_id, target_set):
        """
        Update MLB target set
        
        # Arguments
        target_set_id (String): Target Set Id
        target_set (TargetSet): TargetSet Object
        
        # Returns
        (Object): Spotinst API response 
        """
        target_set = spotinst_mlb.TargetSetRequest(target_set)

        excluded_group_dict = self.exclude_missing(json.loads(target_set.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/targetSet" + target_set_id,
            body=body_json,
            entity_name="mlb target set"
        )
        
        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_target_set(self, target_set_id):
        """
        Gat an MLB target set
        
        # Arguments
        target_set_id (String): Target Set Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/targetSet/" + target_set_id,
            entity_name="mlb target set"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_target_set(self):
        """
        Get all MLB target sets
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/targetSet",
            entity_name="mlb target set"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_target_set(self, target_set_id):
        """
        Delete an MLB target set
        
        # Arguments
        target_set_id (String): Target Set Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/targetSet/" + target_set_id,
            entity_name="mlb target set"
        )

        return response 

    def register_mlb_targets(self, target_set_id, targets):
        """
        Register MLB targets
        
        # Arguments
        target_set_id (String): Target Set Id
        Targets (List[Target]): List of Target Objects
        
        # Returns
        (Object): Spotinst API response 
        """
        targets = spotinst_mlb.TargetsRequest(targets)

        excluded_group_dict = self.exclude_missing(json.loads(targets.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/targetSet/" + target_set_id +
            "/registerTargets",
            body=body_json,
            entity_name="mlb target set"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def deregister_mlb_targets(self, target_set_id, targets):
        """
        Deregister MLB targets
        
        # Arguments
        target_set_id (String): Target Set Id
        Targets (List[Target]): List of Target Objects
        
        # Returns
        (Object): Spotinst API response 
        """
        targets = spotinst_mlb.TargetsRequest(targets)

        excluded_group_dict = self.exclude_missing(json.loads(targets.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/targetSet/" + target_set_id +
            "/deregisterTargets",
            body=body_json,
            entity_name="mlb target set"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def create_mlb_target(self, target):
        """
        Create MLB target
        
        # Arguments
        target (Target): Target Object
        
        # Returns
        (Object): Spotinst API response 
        """
        target = spotinst_mlb.TargetRequest(target)

        excluded_group_dict = self.exclude_missing(json.loads(target.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/target",
            body=body_json,
            entity_name="mlb target"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_target(self, target_id, target):
        """
        Update MLB target
        
        # Arguments
        target_id (String): Target Id
        target (Target): Target Object
        
        # Returns
        (Object): Spotinst API response 
        """
        target = spotinst_mlb.TargetRequest(target)

        excluded_group_dict = self.exclude_missing(json.loads(target.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/target/"+target_id,
            body=body_json,
            entity_name="mlb target"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_target(self, target_id):
        """
        Get MLB target
        
        # Arguments
        target_id (String): Target Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/target/"+target_id,
            entity_name="mlb target"            
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_target(self):
        """
        Get all MLB target
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/target",
            entity_name="mlb target"            
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_target(self, target_id):
        """
        Delete MLB target
        
        # Arguments
        target_id (String): Target Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/target/"+target_id,
            entity_name="mlb target"            
        )

        return response 

    def create_mlb_listener(self, listener):
        """
        Create MLB listener
        
        # Arguments
        listener (Listener): Listener Object
        
        # Returns
        (Object): Spotinst API response 
        """
        listener = spotinst_mlb.ListenerRequest(listener)

        excluded_group_dict = self.exclude_missing(json.loads(listener.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/listener",
            body=body_json,
            entity_name="mlb listener"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_listener(self, listener_id, listener):
        """
        Update MLB listener
        
        # Arguments
        listener_id (String): Listener ID
        listener (Listener): Listener Object
        
        # Returns
        (Object): Spotinst API response 
        """
        listener = spotinst_mlb.ListenerRequest(listener)

        excluded_group_dict = self.exclude_missing(json.loads(listener.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/listener/" + listener_id,
            body=body_json,
            entity_name="mlb listener"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_listener(self, listener_id):
        """
        Get MLB listener
        
        # Arguments
        listener_id (String): Listener ID
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/listener/"+listener_id,
            entity_name="mlb listener"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_listener(self):
        """
        Get all MLB listeners
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/listener",
            entity_name="mlb listener"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_listener(self, listener_id):
        """
        Delete MLB listener
        
        # Arguments
        listener_id (String): Listener ID
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/listener/"+listener_id,
            entity_name="mlb listener"
        )

        return response 

    def create_mlb_routing_rule(self, routing_rule):
        """
        Create MLB routing rule
        
        # Arguments
        routing_rule (RoutingRule): RoutingRule Object
        
        # Returns
        (Object): Spotinst API response 
        """
        routing_rule = spotinst_mlb.RoutingRuleRequest(routing_rule)

        excluded_group_dict = self.exclude_missing(json.loads(routing_rule.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/routingRule",
            body=body_json,
            entity_name="mlb routing rule"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_routing_rule(self, routing_rule_id, routing_rule):
        """
        Update MLB routing rule
        
        # Arguments
        routing_rule_id (String): Routing Rule Id
        routing_rule (RoutingRule): RoutingRule Object
        
        # Returns
        (Object): Spotinst API response 
        """
        routing_rule = spotinst_mlb.RoutingRuleRequest(routing_rule)

        excluded_group_dict = self.exclude_missing(json.loads(routing_rule.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/routingRule/" + routing_rule_id,
            body=body_json,
            entity_name="mlb routing rule"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_routing_rule(self, routing_rule_id):
        """
        Get MLB routing rule
        
        # Arguments
        routing_rule_id (String): Routing Rule Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/routingRule/"+routing_rule_id,
            entity_name="mlb routing rule"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_routing_rule(self):
        """
        Get all MLB routing rule
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/routingRule",
            entity_name="mlb routing rule"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_routing_rule(self, routing_rule_id):
        """
        Delete MLB routing rule
        
        # Arguments
        routing_rule_id (String): Routing Rule Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/routingRule/"+routing_rule_id,
            entity_name="mlb routing rule"
        )

        return response 

    def create_mlb_middleware(self, middleware):
        """
        Create MLB middleware
        
        # Arguments
        middleware (Middleware): Middleware Object
        
        # Returns
        (Object): Spotinst API response 
        """
        middleware = spotinst_mlb.MiddlewareRequest(middleware)

        excluded_group_dict = self.exclude_missing(json.loads(middleware.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_post(
            url=self.__base_lb_url +
            "/middleware",
            body=body_json,
            entity_name="mlb middleware"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def update_mlb_middleware(self, middleware_id, middleware):
        """
        Update MLB middleware
        
        # Arguments
        middleware_id (String): Middleware ID
        middleware (Middleware): Middleware Object
        
        # Returns
        (Object): Spotinst API response 
        """
        middleware = spotinst_mlb.MiddlewareRequest(middleware)

        excluded_group_dict = self.exclude_missing(json.loads(middleware.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        response = self.send_put(
            url=self.__base_lb_url +
            "/middleware/" + middleware_id,
            body=body_json,
            entity_name="mlb middleware"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal 

    def get_mlb_middleware(self, middleware_id):
        """
        Get MLB middleware
        
        # Arguments
        middleware_id (String): Middleware ID
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/middleware/"+middleware_id,
            entity_name="mlb middleware"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal 

    def get_all_mlb_middleware(self):
        """
        Get all MLB middleware
                
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_lb_url +
            "/middleware",
            entity_name="mlb middleware"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal 

    def delete_mlb_middleware(self, middleware_id):
        """
        Delete MLB middleware
        
        # Arguments
        middleware_id (String): Middleware ID
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url=self.__base_lb_url +
            "/middleware/"+middleware_id,
            entity_name="mlb middleware"
        )

        return response 
    # endregion
